#!/usr/bin/python

# Copyright (c) 1999-2002 Red Hat, Inc.  Distributed under GPL.
#
# Author: Mihai Ibanescu <misa@redhat.com>


"""Command-line argument processing module
"""
# $Id: clap.py,v 1.11 2002/04/10 23:06:06 alikins Exp $

import sys

# a bit of a workaround to avoid possible file conflicts
try:
    import popt
except ImportError:
    popt = __import__("up2datepopt")

        
from types import IntType, StringType, FloatType, ListType

# Exception
class ClapError(Exception):
    """"""
    pass

class Clap:
    """
    Command-line argument processing class
    The constructor receives a list of lists in the format:
        short option, long option, info, description, arg description
    and an optional string describing the arguments the program would expect,
    if any.
    Info describes what is expected from this argument. 
        If set to None, the option will be set to 1 if found present (once or
        multiple times)
            For instance, 
            foo --verbose --verbose 
            will set verbose's value to 1
        If set to one of IntType, StringType or FloatType, exactly one value 
        of the specified type is expected (and the last one wins).
            For instance,
            foo --foo=1 --foo=2
            will set foo's value to 2
        If set to a list, a list of values is returned.
        If the list is empty, it will behave like setting the option to None 
        and returning all occurences of the option in a list. This is useful
        if the number of flags is important (like increasing the verbosity
        level).
        If the list is not empty, the first item in the list has to be one
        IntType, StringType or FloatType; the return is a list with items of
        that type.
            For instance,
            foo --foo=1 --foo=2
            will set foo's value to [1, 2], if foo's info field was decribed 
            as [IntType]
    If the string "help" replaces one of the table's lines, autohelp will be
    generated too. Using --help or -? will print all the available options in
    a nice format; --usage will shortly describe the arguments.
    
    The process() method expects a list of arguments in sys.argv style. The
    first item of the list should be the name of the program.
    
    Two hashes allows one to retrieve the value of an object after processing.
    hashShort will map short names to values
    hashLong will map long names to values
    getShort() and getLong() are methods returning items from the hashes.

    Here is an example of how to use the class

    table = [
        "help",
        ['v', 'verbose',     [], 'Increase verbosity'],
        ['d', 'dir',         StringType, 
                'Process packages from this directory (current dir)', 'DIR'],
        ['c', 'channel',     [StringType],
                'Manage this channel'],
        ['n', 'count',       IntType,
                'Process this number of headers per call'],
        ['l', 'list',        None,
                'Only list the specified channels'],
    ]
    # Define the object
    argObj = Clap(table, "Files")
    argObj.process(arglist)
    
    # Process the arguments
    print argObj.getLong("channel")
    """

    def __init__(self, table, argsDescription = None, aliasing = None,
                poptname = None):
        """Constructor"""
        if type(table) is not ListType:
            raise ClapError("Wrong type for the table: expected list, got %s" %
                type(table))
        # Hashes to map all the options
        self.hashShort = {}
        self.hashLong = {}
        # The popt table
        self.table = []
        # Arguments
        self.args = []
        # Arguments description, if any
        self.argsDescription = argsDescription
        # Do we allow for aliasing?
        self.aliasing = aliasing
        # Name to look up in the popt config files (/etc/popt or ~/.popt)
        self.poptname = poptname
        # For sanity checks (no option is present twice)
        shortNames = {}
        longNames = {}
        # For internal use: is this value an array?
        self.__valueType = []
        # The context
        self.ctx = None
        # Build the corresponding popt table
        for o in table:
            if o == "help":
                self.table.append(popt.POPT_AUTOHELP)
                self.__valueType.append(None)
                continue
            if type(o) is not ListType:
                raise ClapError('Expected a list or "help"; received %s' %
                    type(o))
            if len(o) < 4:
                raise ClapError("Expected at least 4 arguments; %s received" %
                    len(o))
            entry = []
            # Long name
            if not o[1]:
                entry.append(None)
            else:
                if longNames.has_key(o[1]):
                    raise ClapError("Duplicate definition for option %s" %
                        o[1])
                entry.append(o[1])
                longNames[o[1]] = None
            # Short name
            if not o[0]:
                entry.append(None)
            else:
                if shortNames.has_key(o[0]):
                    raise ClapError("Duplicate definition for option %s" %
                        o[0])
                entry.append(o[0])
                shortNames[o[0]] = None
            # Type and val
            val = 0
            valueType = o[2]
            if o[2] is None:
                entry.append(popt.POPT_ARG_NONE)
            elif o[2] in (IntType, StringType, FloatType):
                entry.append(_pyType2poptType(o[2]))
            elif type(o[2]) is ListType:
                # They want a list returned. Fine
                val = len(self.table) + 1
                if len(o[2]) == 0:
                    # Empty list
                    entry.append(popt.POPT_ARG_NONE)
                elif o[2][0] in (IntType, StringType, FloatType):
                    entry.append(_pyType2poptType(o[2][0]))
                else:
                    raise ClapError("Unknown type")
            else:
                raise ClapError("Unknown type %s" % type(o[2]))
            entry.append(val)
            entry.append(o[3])
            self.__valueType.append(valueType)
            if len(o) > 4:
                entry.append(o[4])
            self.table.append(entry)
    
    def process(self, arglist):
        """Calling this method will do all the dirty work"""
        # The name to look up in the python config files
        poptname = self.poptname or arglist[0]
        self.ctx = popt.getContext(poptname, arglist, self.table)
        if self.argsDescription:
            self.ctx.setOtherOptionHelp(self.argsDescription)
        if self.aliasing:
            self.ctx.readDefaultConfig()
        resultlist = map(lambda x: [], self.table)
        while 1:
            rc = self.ctx.getNextOpt()
            if (rc == -1):
                break
            if (rc < 0):
                sys.stderr.write("Unknown option %s\n" % self.ctx.badOption())
                self.ctx.printUsage()
                sys.exit(-1)
            resultlist[rc - 1].append(self.ctx.getOptValue())
        values = self.ctx.getOptValues()
        self.args = self.ctx.getArgs() or []
        result = []
        for i in range(len(resultlist)):
            # Do they want some incremental stuff?
            if type(self.__valueType[i]) is ListType:
                result.append(resultlist[i])
            else:
                result.append(values[i])
                
        for i in range(len(self.table)):
            # Get the option
            opt = self.table[i]
            if type(opt) is StringType:
                # Autohelp
                continue
            if opt[0]:
                # hash by long value
                self.hashLong[opt[0]] = result[i]
            if opt[1]:
                # hash by short value
                self.hashShort[opt[1]] = result[i]

    def getShort(self, name):
        """Retrieve an option's value by short name"""
        if not self.hashShort.has_key(name):
            raise ClapError("Short option '%s' not defined" % name)
        return self.hashShort[name]

    def getLong(self, name):
        """Retrieve an option's value by long name"""
        if not self.hashLong.has_key(name):
            raise ClapError("Long option '%s' not defined" % name)
        return self.hashLong[name]
    

def _pyType2poptType(arg):
    if arg == IntType:
        return popt.POPT_ARG_INT
    if arg == StringType:
        return popt.POPT_ARG_STRING
    if arg == FloatType: 
        return popt.POPT_ARG_FLOAT
    raise ClapError("Unknown option")
    

if __name__ == '__main__':
    arglist = sys.argv
    # Initialize a command-line processing object with a table of options
    table = [
        "help",
        ['v', 'verbose',     [], 'Increase verbosity'],
        ['d', 'dir',         StringType, 
                'Process packages from this directory (current dir)', 'DIR'],
        ['f', 'conffile',    StringType,
                'Configuration file (~/.rhn_package_manager)', 'FILE'],
        ['c', 'channel',     [StringType],
                'Manage this channel'],
        ['n', 'count',       IntType,
                'Process this number of headers per call'],
        ['l', 'list',        None,
                'Only list the specified channels'],
        ['s', 'sync',        None,
                'Check if in sync with the server'],
        ['k', 'createdir',   None,
                'Creates directories for channels'],
        ['p', 'printconf',   None,
                'Print the configuration and exit'],
        ['r', 'reldir',      None,
                'Relative dir to associate with the file'],
        ['x', None,          None,
                'Test short option alone'],
    ]
    argObj = Clap(table, "Files")
    argObj.process(arglist)
    print "Dir: %s" % argObj.getLong('dir')
    print "Channels: %s" % argObj.getLong('channel')
    print "Short: %s" % argObj.hashShort
    print "Long: %s" % argObj.hashLong
    print "Verbose: %s" % argObj.getLong("verbose")


