#!/usr/bin/python
#
# Client code for Update Agent
# Copyright (c) 1999-2001 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com
#         Cristian Gafton <gafton@redhat.com>
#

import up2dateLog
from translate import _, N_, cat

log = up2dateLog.initLog()

class Error:
    """base class for errors"""
    def __init__(self, errmsg):
        self.errmsg = errmsg

    def __repr__(self):
        log.log_me(self.errmsg)
        return self.errmsg
    
class FileError(Error):
    """error to report when we encounter file errors (missing files/dirs,
       lack of permissions, quoat issues, etc"""
    def __repr__(self):
        msg = _("Disk error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class RpmError(Error):
    """rpm itself raised an error condition"""
    def __repr__(self):
        msg = _("RPM error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class RpmInstallError(Error):
    """Raise when a package fails to install properly"""
    def __init__(self, msg, pkg = None):
        self.errmsg = msg
        self.pkg = pkg
    def __repr__(self):
        msg = _("There was a fatal error installing the package:\n" + self.errmsg)
        log.log_me(msg)
        return msg
    

class PasswordError(Error):
    """Raise when the server responds with that a password is incorrect"""
    def __repr__(self):
        msg = _("Password error. The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class ConflictError(Error):
    """Raise when a rpm transaction set has a package conflict"""
    def __init__(self, msg, rc=None):
        self.rc = rc
        self.errmsg = msg 
    def __repr__(self):
        msg = _("RPM package conflict error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class FileConflictError(Error):
    """Raise when a rpm tranaction set has a file conflict"""
    def __init__(self, msg, rc=None):
        self.rc = rc
        self.errmsg = msg
    def __repr__(self):
        msg = _("RPM file conflict error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg
    
class DependencyError(Error):
    """Raise when a rpm transaction set has a dependency error"""
    def __init__(self, msg, deps=None):
        self.errmsg = msg
        # just tag on the whole deps tuple, so we have plenty of info to play with
        self.deps = deps
        
    def __repr__(self):
        msg = _("RPM dependency error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class UnsolvedDependencyError(Error):
    """Raise when we have a dependency that the server can not find"""
    def __init__(self, msg, dep=None, pkgs=None):
        self.errmsg = msg
        self.dep = dep
        self.pkgs = pkgs 
    def __repr__(self):
        msg = _("RPM dependency error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class DependencySenseError(Error):
    """Raise when a rpm transaction set has a dependency sense we don't understand"""
    def __init__(self, msg, sense=None):
        self.errmsg = msg
        self.sense = sense
    def __repr__(self):
        msg = _("RPM dependency error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class SkipListError(Error):
    """Raise when all the packages you want updated are on a skip list"""
    def __init__(self, msg, pkglist=None):
	self.errmsg = msg
	self.pkglist = pkglist 
    def __repr__(self):
        msg = _("Package Skip List error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class FileConfigSkipListError(Error):
    """Raise when all the packages you want updated are skip because of config or file skip list"""
    def __init__(self, msg, pkglist=None):
        self.errmsg = msg
        self.pkglist = None
    def __repr__(self):
        msg = _("File Skip List or config file overwrite error.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg


class CommunicationError(Error):
    """Indicates a problem doing xml-rpc http communication with the server"""
    def __repr__(self):
        msg =  _("Error communicating with server.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class FileNotFoundError(Error):
    """Raise when a package or header that is requested returns a 404 error code"""
    def __repr__(self):
        msg =  _("File Not Found: \n") + self.errmsg
        log.log_me(msg)
        return msg


class DelayError(Error):
    """Raise when the expected response from a xml-rpc call exceeds a timeout"""
    def __repr__(self):
        msg =  _("Delay error from server.  The message was:\n") + self.errmsg
        log.log_me(msg)
        return msg

class RpmRemoveSkipListError(Error):
    """Raise when we try to remove a package on the RemoveSkipList"""
    def __repr__(self):
        msg = _("Could not remove package \"%s\". It was on the RemoveSkipList") % self.errmsg
        log.log_me(msg)
        return msg

class RpmRemoveError(Error):
    """Raise when we can't remove a package for some reason (failed deps, etc)"""
    def __init__(self, args):
        self.args = args
        self.errmsg = ""
        for key in self.args.keys():
            self.errmsg = self.errmsg + "%s failed because of %s\n" % (key, self.args[key])     
    def __repr__(self):
        return self.errmsg

class GPGInstallationError(Error):
    """Raise when we we detect that the GPG is not installed properly"""
    def __repr__(self):
        msg = _("GPG is not installed properly.") 
        log.log_me(msg)
        return msg

class GPGKeyringError(Error):
    """Raise when we we detect that the gpg keyring for the user does not have the Red Hat Key installed"""
    def __repr__(self):
        msg = _("GPG keyring does not include the Red Hat, Inc. public package-signing key")
        log.log_me(msg)
        return msg

class GPGVerificationError(Error):
    """Raise when we fail to verify a package is signed with a gpg signature"""
    def __init__(self, msg):
        self.errmsg = msg
        self.pkg = msg
    def __repr__(self):
        msg = _("The package %s failed its gpg signature verification. This means the package is corrupt." % self.errmsg)
        log.log_me(msg)
        return msg

class GPGVerificationUnsignedPackageError(Error):
    """Raise when a package that is supposed to be verified has no gpg signature"""
    def __init__(self, msg):
        self.errmsg = msg
        self.pkg = msg
    def __repr__(self):
        msg = _("The package %s does not have a GPG signature.\n") %  self.errmsg
        log.log_me(msg)
        return msg

class OutOfSpaceError(Error):
    def __init__(self, totalSize, freeDiskSpace):
        self.ts = totalSize
        self.fds = freeDiskSpace
        self.errmsg = "The total size of the selected packages (%d kB) exceeds your free disk space (%d kB)." % (self.ts, self.fds)

    def __repr__(self):
        return self.errmsg

